@@ -1,11 +1,11 @@
-__version__ = "22.1.3"
+__version__ = "22.1.5"
 
 PROJECT_NAME = "galaxy-tool-util"
 PROJECT_OWNER = PROJECT_USERAME = "galaxyproject"
 PROJECT_URL = "https://github.com/galaxyproject/galaxy"
-PROJECT_AUTHOR = 'Galaxy Project and Community'
-PROJECT_DESCRIPTION = 'Galaxy Tool and Tool Dependency Utilities'
-PROJECT_EMAIL = 'galaxy-committers@lists.galaxyproject.org'
+PROJECT_AUTHOR = "Galaxy Project and Community"
+PROJECT_DESCRIPTION = "Galaxy Tool and Tool Dependency Utilities"
+PROJECT_EMAIL = "galaxy-committers@lists.galaxyproject.org"
 RAW_CONTENT_URL = "https://raw.github.com/{}/{}/master/".format(
     PROJECT_USERAME, PROJECT_NAME
 )
