@@ -1,5 +1,6 @@
 """Abstraction around cwltool and related libraries for loading a CWL artifact."""
 import os
+import tempfile
 from collections import namedtuple
 
 from .cwltool_deps import (
@@ -32,19 +33,26 @@ class SchemaLoader:
         loading_context.do_validate = self._validate
         loading_context.loader = self.raw_document_loader
         loading_context.do_update = True
+        loading_context.relax_path_checks = True
         return loading_context
 
     def raw_process_reference(self, path, loading_context=None):
-        path = os.path.abspath(path)
-        uri = "file://" + path
-        loading_context = loading_context or self.loading_context()
-        if REWRITE_EXPRESSIONS and not uri.endswith(".galaxy"):
-            galaxy_path = os.path.abspath(path) + ".galaxy"
-            from cwl_utils import etools_to_clt
-            etools_to_clt.main([path, galaxy_path])
-            galaxy_uri = "file://" + galaxy_path
-            uri = galaxy_uri
-        loading_context, process_object, uri = load_tool.fetch_document(uri, loadingContext=loading_context)
+        with tempfile.TemporaryDirectory() as output_dir:
+            suffix = ''
+            if '#' in path:
+                path, suffix = path.split('#')
+            processed_path = os.path.join(output_dir, os.path.basename(path))
+            path = os.path.abspath(path)
+            uri = f"file://{path}"
+            loading_context = loading_context or self.loading_context()
+            if REWRITE_EXPRESSIONS:
+                from cwl_utils import cwl_expression_refactor
+                exit_code = cwl_expression_refactor.main([output_dir, path, '--skip-some1', '--skip-some2'])
+                if exit_code == 0:
+                    uri = f"file://{processed_path}"
+            if suffix:
+                uri = f"{uri}#{suffix}"
+            loading_context, process_object, uri = load_tool.fetch_document(uri, loadingContext=loading_context)
         return RawProcessReference(loading_context, process_object, uri)
 
     def raw_process_reference_for_object(self, process_object, uri=None, loading_context=None):
