@@ -8,73 +8,107 @@ import sys
 import tarfile
 import threading
 from io import BytesIO
+from typing import (
+    List,
+    TYPE_CHECKING,
+)
 
 import packaging.version
 import requests
 
+if TYPE_CHECKING:
+    from galaxy.tool_util.deps.container_resolvers import ResolutionCache
+
 log = logging.getLogger(__name__)
 
 QUAY_REPOSITORY_API_ENDPOINT = 'https://quay.io/api/v1/repository'
 BUILD_NUMBER_REGEX = re.compile(r'\d+$')
-PARSED_TAG = collections.namedtuple('ParsedTag', 'tag version build_string build_number')
-QUAY_IO_TIMEOUT = 10
+PARSED_TAG = collections.namedtuple('PARSED_TAG', 'tag version build_string build_number')
+MULLED_SOCKET_TIMEOUT = 12
+QUAY_VERSIONS_CACHE_EXPIRY = 300
+NAMESPACE_HAS_REPO_NAME_KEY = "galaxy.tool_util.deps.container_resolvers.mulled.util:namespace_repo_names"
+TAG_CACHE_KEY = "galaxy.tool_util.deps.container_resolvers.mulled.util:tag_cache"
 
 
 def create_repository(namespace, repo_name, oauth_token):
     assert oauth_token
-    headers = {'Authorization': 'Bearer %s' % oauth_token}
+    headers = {'Authorization': f'Bearer {oauth_token}'}
     data = {
         "repository": repo_name,
         "namespace": namespace,
         "description": "",
         "visibility": "public",
     }
-    requests.post("https://quay.io/api/v1/repository", json=data, headers=headers, timeout=QUAY_IO_TIMEOUT)
+    requests.post("https://quay.io/api/v1/repository", json=data, headers=headers, timeout=MULLED_SOCKET_TIMEOUT)
 
 
-def quay_versions(namespace, pkg_name):
+def quay_versions(namespace, pkg_name, session=None):
     """Get all version tags for a Docker image stored on quay.io for supplied package name."""
-    data = quay_repository(namespace, pkg_name)
+    data = quay_repository(namespace, pkg_name, session=session)
 
     if 'error_type' in data and data['error_type'] == "invalid_token":
         return []
 
     if 'tags' not in data:
-        raise Exception("Unexpected response from quay.io - no tags description found [%s]" % data)
+        raise Exception(f"Unexpected response from quay.io - no tags description found [{data}]")
 
     return [tag for tag in data['tags'].keys() if tag != 'latest']
 
 
-def quay_repository(namespace, pkg_name):
+def quay_repository(namespace, pkg_name, session=None):
     assert namespace is not None
     assert pkg_name is not None
-    url = 'https://quay.io/api/v1/repository/{}/{}'.format(namespace, pkg_name)
-    response = requests.get(url, timeout=QUAY_IO_TIMEOUT)
+    url = f'https://quay.io/api/v1/repository/{namespace}/{pkg_name}'
+    if not session:
+        session = requests.session()
+    response = session.get(url, timeout=MULLED_SOCKET_TIMEOUT)
     data = response.json()
     return data
 
 
-def _namespace_has_repo_name(namespace, repo_name, resolution_cache):
+def _get_namespace(namespace: str) -> List[str]:
+    next_page = None
+    repo_names = []
+    repos_headers = {"Accept-encoding": "gzip", "Accept": "application/json"}
+    while True:
+        repos_parameters = {"public": "true", "namespace": namespace, "next_page": next_page}
+        repos_response = requests.get(
+            QUAY_REPOSITORY_API_ENDPOINT, headers=repos_headers, params=repos_parameters, timeout=MULLED_SOCKET_TIMEOUT
+        )
+        repos_response_json = repos_response.json()
+        repos = repos_response_json["repositories"]
+        repo_names += [r["name"] for r in repos]
+        next_page = repos_response_json.get("next_page")
+        if not next_page:
+            break
+    return repo_names
+
+
+def _namespace_has_repo_name(namespace: str, repo_name: str, resolution_cache: "ResolutionCache") -> bool:
     """
     Get all quay containers in the biocontainers repo
     """
-    cache_key = "galaxy.tool_util.deps.container_resolvers.mulled.util:namespace_repo_names"
-    if resolution_cache is not None and cache_key in resolution_cache:
-        repo_names = resolution_cache.get(cache_key)
-    else:
-        repos_parameters = {'public': 'true', 'namespace': namespace}
-        repos_headers = {'Accept-encoding': 'gzip', 'Accept': 'application/json'}
-        repos_response = requests.get(
-            QUAY_REPOSITORY_API_ENDPOINT, headers=repos_headers, params=repos_parameters, timeout=QUAY_IO_TIMEOUT)
-
-        repos = repos_response.json()['repositories']
-        repo_names = [r["name"] for r in repos]
-        if resolution_cache is not None:
-            resolution_cache[cache_key] = repo_names
+    # resolution_cache.mulled_resolution_cache is the persistent variant of the resolution cache
+    preferred_resolution_cache = resolution_cache.mulled_resolution_cache or resolution_cache
+    cache_key = NAMESPACE_HAS_REPO_NAME_KEY
+    if preferred_resolution_cache is not None:
+        try:
+            cached_namespace = preferred_resolution_cache.get(cache_key)
+            if cached_namespace:
+                return repo_name in cached_namespace
+        except KeyError:
+            # preferred_resolution_cache may be a beaker Cache instance, which
+            # raises KeyError if key is not present on `.get`
+            pass
+    repo_names = _get_namespace(namespace)
+    if preferred_resolution_cache is not None:
+        preferred_resolution_cache[cache_key] = repo_names
     return repo_name in repo_names
 
 
-def mulled_tags_for(namespace, image, tag_prefix=None, resolution_cache=None):
+def mulled_tags_for(
+    namespace, image, tag_prefix=None, resolution_cache=None, session=None, expire=QUAY_VERSIONS_CACHE_EXPIRY
+):
     """Fetch remote tags available for supplied image name.
 
     The result will be sorted so newest tags are first.
@@ -83,11 +117,16 @@ def mulled_tags_for(namespace, image, tag_prefix=None, resolution_cache=None):
         # Following check is pretty expensive against biocontainers... don't even bother doing it
         # if can't cache the response.
         if not _namespace_has_repo_name(namespace, image, resolution_cache):
-            log.debug("skipping mulled_tags_for [%s] no repository" % image)
+            log.info(f"skipping mulled_tags_for [{image}] no repository")
             return []
 
-    cache_key = "galaxy.tool_util.deps.container_resolvers.mulled.util:tag_cache"
+    cache_key = TAG_CACHE_KEY
     if resolution_cache is not None:
+        if resolution_cache.mulled_resolution_cache is not None:
+            # Use persistent cache if possible. Since tags query is lightweight use a relatively short expiry time.
+            resolution_cache = resolution_cache.mulled_resolution_cache._get_cache(
+                "mulled_tag_cache", {"expire": expire}
+            )
         if cache_key not in resolution_cache:
             resolution_cache[cache_key] = collections.defaultdict(dict)
         tag_cache = resolution_cache.get(cache_key)
@@ -95,13 +134,14 @@ def mulled_tags_for(namespace, image, tag_prefix=None, resolution_cache=None):
         tag_cache = collections.defaultdict(dict)
 
     tags_cached = False
-    if namespace in tag_cache:
-        if image in tag_cache[namespace]:
-            tags = tag_cache[namespace][image]
-            tags_cached = True
+    try:
+        tags = tag_cache[namespace][image]
+        tags_cached = True
+    except KeyError:
+        pass
 
     if not tags_cached:
-        tags = quay_versions(namespace, image)
+        tags = quay_versions(namespace, image, session)
         tag_cache[namespace][image] = tags
 
     if tag_prefix is not None:
@@ -158,16 +198,17 @@ def build_target(package_name, version=None, build=None, tag=None):
         assert build is None
         version, build = split_tag(tag)
 
-    return Target(package_name, version, build, package_name)
+    # conda package and quay image names are lowercase
+    return Target(package_name.lower(), version, build, package_name)
 
 
 def conda_build_target_str(target):
     rval = target.package_name
     if target.version:
-        rval += "=%s" % target.version
+        rval += f"={target.version}"
 
         if target.build:
-            rval += "=%s" % target.build
+            rval += f"={target.build}"
 
     return rval
 
@@ -181,10 +222,10 @@ def _simple_image_name(targets, image_build=None):
             # Special case image_build == "0", which has been built without a suffix
             print("WARNING: Hard-coding image build instead of using Conda build - this is not recommended.")
             build = image_build
-        suffix += ":%s" % target.version
+        suffix += f":{target.version}"
         if build is not None:
-            suffix += "--%s" % build
-    return "{}{}".format(target.package_name, suffix)
+            suffix += f"--{build}"
+    return f"{target.package_name}{suffix}"
 
 
 def v1_image_name(targets, image_build=None, name_override=None):
@@ -220,8 +261,8 @@ def v1_image_name(targets, image_build=None, name_override=None):
         requirements_buffer = "\n".join(map(conda_build_target_str, targets_order))
         m = hashlib.sha1()
         m.update(requirements_buffer.encode())
-        suffix = "" if not image_build else ":%s" % image_build
-        return "mulled-v1-{}{}".format(m.hexdigest(), suffix)
+        suffix = "" if not image_build else f":{image_build}"
+        return f"mulled-v1-{m.hexdigest()}{suffix}"
 
 
 def v2_image_name(targets, image_build=None, name_override=None):
@@ -281,19 +322,19 @@ def v2_image_name(targets, image_build=None, name_override=None):
             build_suffix = ""
         elif version_hash_str:
             # tagged verson is <version_hash>-<build>
-            build_suffix = "-%s" % image_build
+            build_suffix = f"-{image_build}"
         else:
             # tagged version is simply the build
             build_suffix = image_build
         suffix = ""
         if version_hash_str or build_suffix:
-            suffix = ":{}{}".format(version_hash_str, build_suffix)
-        return "mulled-v2-{}{}".format(package_hash.hexdigest(), suffix)
+            suffix = f":{version_hash_str}{build_suffix}"
+        return f"mulled-v2-{package_hash.hexdigest()}{suffix}"
 
 
 def get_file_from_recipe_url(url):
     """Downloads file at url and returns tarball"""
-    r = requests.get(url)
+    r = requests.get(url, timeout=MULLED_SOCKET_TIMEOUT)
     return tarfile.open(mode="r:bz2", fileobj=BytesIO(r.content))
 
 
