@@ -1,6 +1,5 @@
 import os
-
-from six.moves import shlex_quote
+import shlex
 
 DEFAULT_WORKING_DIRECTORY = None
 DEFAULT_SINGULARITY_COMMAND = "singularity"
@@ -23,18 +22,31 @@ def pull_mulled_singularity_command(docker_image_identifier,
     )
     save_path = docker_image_identifier
     if namespace:
-        prefix = "docker://quay.io/%s/" % namespace
+        prefix = f"docker://quay.io/{namespace}/"
         if docker_image_identifier.startswith(prefix):
             save_path = docker_image_identifier[len(prefix):]
     command_parts.extend(["build", os.path.join(cache_directory, save_path), docker_image_identifier])
     return command_parts
 
 
+def pull_singularity_command(image_identifier: str,
+                             cache_path: str,
+                             singularity_cmd: str = DEFAULT_SINGULARITY_COMMAND,
+                             sudo: bool = DEFAULT_SUDO,
+                             sudo_cmd: str = DEFAULT_SUDO_COMMAND):
+    # Make sure cache dir exists
+    dirname = os.path.dirname(os.path.normpath(cache_path))
+    os.makedirs(dirname, exist_ok=True)
+    command_parts = _singularity_prefix(singularity_cmd, sudo, sudo_cmd)
+    command_parts.extend(["build", cache_path, image_identifier])
+    return command_parts
+
+
 def build_singularity_run_command(
     container_command,
     image,
-    volumes=[],
-    env=[],
+    volumes=None,
+    env=None,
     working_directory=DEFAULT_WORKING_DIRECTORY,
     singularity_cmd=DEFAULT_SINGULARITY_COMMAND,
     run_extra_arguments=DEFAULT_RUN_EXTRA_ARGUMENTS,
@@ -43,13 +55,15 @@ def build_singularity_run_command(
     guest_ports=False,
     container_name=None
 ):
+    volumes = volumes or []
+    env = env or []
     command_parts = []
     # http://singularity.lbl.gov/docs-environment-metadata
     home = None
     for (key, value) in env:
         if key == 'HOME':
             home = value
-        command_parts.extend(["SINGULARITYENV_{}={}".format(key, value)])
+        command_parts.extend([f"SINGULARITYENV_{key}={value}"])
     command_parts += _singularity_prefix(
         singularity_cmd=singularity_cmd,
         sudo=sudo,
@@ -60,11 +74,11 @@ def build_singularity_run_command(
     for volume in volumes:
         command_parts.extend(["-B", str(volume)])
     if home is not None:
-        command_parts.extend(["--home", "{}:{}".format(home, home)])
+        command_parts.extend(["--home", f"{home}:{home}"])
     if run_extra_arguments:
         command_parts.append(run_extra_arguments)
     full_image = image
-    command_parts.append(shlex_quote(full_image))
+    command_parts.append(shlex.quote(full_image))
     command_parts.append(container_command)
     return " ".join(command_parts)
 
@@ -83,4 +97,5 @@ def _singularity_prefix(
     return command_parts
 
 
-__all__ = ("build_singularity_run_command", "pull_mulled_singularity_command")
+__all__ = ("build_singularity_run_command", "pull_mulled_singularity_command",
+           "pull_singularity_command")
