@@ -1,10 +1,14 @@
+import fnmatch
 import os
+import re
 from abc import (
     ABCMeta,
     abstractmethod
 )
+from os.path import join
+from typing import Dict, List, Optional
 
-
+from galaxy.util.path import safe_walk
 from .util import _parse_name
 
 NOT_IMPLEMENTED_MESSAGE = "Galaxy tool format does not yet support this tool feature."
@@ -14,7 +18,7 @@ class ToolSource(metaclass=ABCMeta):
     """ This interface represents an abstract source to parse tool
     information from.
     """
-    default_is_multi_byte = False
+    language: str
 
     @abstractmethod
     def parse_id(self):
@@ -55,11 +59,16 @@ class ToolSource(metaclass=ABCMeta):
     def parse_description(self):
         """ Parse a description for tool. Longer than name, shorted than help. """
 
-    def parse_is_multi_byte(self):
-        """ Parse is_multi_byte from tool - TODO: figure out what this is and
-        document.
-        """
-        return self.default_is_multi_byte
+    def parse_edam_operations(self) -> List[str]:
+        """Parse list of edam operation codes."""
+        return []
+
+    def parse_edam_topics(self) -> List[str]:
+        """Parse list of edam topic codes."""
+        return []
+
+    def parse_xrefs(self) -> List[Dict[str, str]]:
+        """Parse list of external resource URIs and types."""
 
     def parse_display_interface(self, default):
         """ Parse display_interface - fallback to default for the tool type
@@ -111,7 +120,8 @@ class ToolSource(metaclass=ABCMeta):
         return ["TMPDIR", "TMP", "TEMP"]
 
     def parse_docker_env_pass_through(self):
-        return ["GALAXY_SLOTS", "HOME", "_GALAXY_JOB_HOME_DIR", "_GALAXY_JOB_TMP_DIR"] + self.parse_tmp_directory_vars()
+        return ["GALAXY_SLOTS", "GALAXY_MEMORY_MB", "GALAXY_MEMORY_MB_PER_SLOT", "HOME",
+                "_GALAXY_JOB_HOME_DIR", "_GALAXY_JOB_TMP_DIR"] + self.parse_tmp_directory_vars()
 
     @abstractmethod
     def parse_interpreter(self):
@@ -167,6 +177,10 @@ class ToolSource(metaclass=ABCMeta):
         """
         return False
 
+    def parse_required_files(self) -> Optional['RequiredFiles']:
+        """ Parse explicit RequiredFiles object or return None to let Galaxy decide implicit logic."""
+        return None
+
     @abstractmethod
     def parse_requirements_and_containers(self):
         """ Return pair of ToolRequirement and ContainerDescription lists. """
@@ -221,12 +235,23 @@ class ToolSource(metaclass=ABCMeta):
         """ Return tool profile version as Galaxy major e.g. 16.01 or 16.04.
         """
 
+    @abstractmethod
+    def parse_license(self):
+        """Return license corresponding to tool wrapper."""
+
     @abstractmethod
     def parse_python_template_version(self):
         """
         Return minimum python version that the tool template has been developed against.
         """
 
+    def parse_creator(self):
+        """Return list of metadata relating to creator/author of tool.
+
+        Result should be list of schema.org data model Person or Organization objects.
+        """
+        return []
+
     @property
     def macro_paths(self):
         return []
@@ -247,9 +272,9 @@ class ToolSource(metaclass=ABCMeta):
     def __str__(self):
         source_path = self.source_path
         if source_path:
-            as_str = '{}[{}]'.format(self.__class__.__name__, source_path)
+            as_str = f'{self.__class__.__name__}[{source_path}]'
         else:
-            as_str = '%s[In-memory]' % (self.__class__.__name__)
+            as_str = f'{self.__class__.__name__}[In-memory]'
         return as_str
 
 
@@ -274,7 +299,7 @@ class PageSource(metaclass=ABCMeta):
         return None
 
     @abstractmethod
-    def parse_input_sources(self):
+    def parse_input_sources(self) -> List:
         """ Return a list of InputSource objects. """
 
 
@@ -362,6 +387,7 @@ class InputSource(metaclass=ABCMeta):
 
 
 class TestCollectionDef:
+    __test__ = False  # Prevent pytest from discovering this class (issue #12071)
 
     def __init__(self, attrib, name, collection_type, elements):
         self.attrib = attrib
@@ -416,7 +442,7 @@ class TestCollectionDef:
 
         def element_from_dict(element_dict):
             if "element_definition" not in element_dict:
-                raise Exception("Invalid element_dict %s" % element_dict)
+                raise Exception(f"Invalid element_dict {element_dict}")
             element_def = element_dict["element_definition"]
             if element_def.get("model_class", None) == "TestCollectionDef":
                 element_def = TestCollectionDef.from_dict(element_def)
@@ -440,7 +466,57 @@ class TestCollectionDef:
         return inputs
 
 
+class RequiredFiles:
+
+    def __init__(self, includes: List[Dict], excludes: List[Dict], extend_default_excludes: bool):
+        self.includes = includes
+        self.excludes = excludes
+        self.extend_default_excludes = extend_default_excludes
+
+    @staticmethod
+    def from_dict(as_dict):
+        extend_default_excludes: bool = as_dict.get("extend_default_excludes", True)
+        includes: List = as_dict.get("includes", [])
+        excludes: List = as_dict.get("excludes", [])
+        return RequiredFiles(includes, excludes, extend_default_excludes)
+
+    def find_required_files(self, tool_directory: str) -> List[str]:
+
+        def matches(ie_list: List, rel_path: str):
+            for ie_item in ie_list:
+                ie_item_path = ie_item["path"]
+                ie_item_type = ie_item.get("path_type", "literal")
+                if ie_item_type == "literal":
+                    if rel_path == ie_item_path:
+                        return True
+                elif ie_item_type == "prefix":
+                    if rel_path.startswith(ie_item_path):
+                        return True
+                elif ie_item_type == "glob":
+                    if fnmatch.fnmatch(rel_path, ie_item_path):
+                        return True
+                else:
+                    if re.match(ie_item_path, rel_path) is not None:
+                        return True
+            return False
+
+        excludes = self.excludes
+        if self.extend_default_excludes:
+            excludes.append({"path": "tool-data", "path_type": "prefix"})
+            excludes.append({"path": "test-data", "path_type": "prefix"})
+            excludes.append({"path": ".hg", "path_type": "prefix"})
+
+        files: List[str] = []
+        for (dirpath, _, filenames) in safe_walk(tool_directory):
+            for filename in filenames:
+                rel_path = join(dirpath, filename).replace(tool_directory + os.path.sep, '')
+                if matches(self.includes, rel_path) and not matches(self.excludes, rel_path):
+                    files.append(rel_path)
+        return files
+
+
 class TestCollectionOutputDef:
+    __test__ = False  # Prevent pytest from discovering this class (issue #12071)
 
     def __init__(self, name, attrib, element_tests):
         self.name = name
@@ -454,7 +530,7 @@ class TestCollectionOutputDef:
     def from_dict(as_dict):
         return TestCollectionOutputDef(
             name=as_dict["name"],
-            attrib=as_dict["attributes"],
+            attrib=as_dict.get("attributes", {}),
             element_tests=as_dict["element_tests"],
         )
 
