@@ -1,30 +1,35 @@
-from collections import OrderedDict
+from typing import List
 
 from galaxy.util.dictifiable import Dictifiable
 from .output_actions import ToolOutputActionGroup
-from .output_collection_def import dataset_collector_descriptions_from_output_dict
+from .output_collection_def import dataset_collector_descriptions_from_output_dict, DatasetCollectionDescription
 
 
 class ToolOutputBase(Dictifiable):
 
-    def __init__(self, name, label=None, filters=None, hidden=False):
+    def __init__(self, name, label=None, filters=None, hidden=False, from_expression=None):
         super().__init__()
         self.name = name
         self.label = label
         self.filters = filters or []
         self.hidden = hidden
         self.collection = False
+        self.from_expression = from_expression
 
     def to_dict(self, view='collection', value_mapper=None, app=None):
         return super().to_dict(view=view, value_mapper=value_mapper)
 
+    @property
+    def output_discover_patterns(self) -> List[str]:
+        return []
+
 
 class ToolOutput(ToolOutputBase):
     """
     Represents an output datasets produced by a tool. For backward
     compatibility this behaves as if it were the tuple::
 
-      (format, metadata_source, parent)
+    (format, metadata_source, parent)
     """
 
     dict_collection_visible_keys = ['name', 'format', 'label', 'hidden', 'output_type', 'format_source',
@@ -32,8 +37,8 @@ class ToolOutput(ToolOutputBase):
 
     def __init__(self, name, format=None, format_source=None, metadata_source=None,
                  parent=None, label=None, filters=None, actions=None, hidden=False,
-                 implicit=False):
-        super().__init__(name, label=label, filters=filters, hidden=hidden)
+                 implicit=False, from_expression=None):
+        super().__init__(name, label=label, filters=filters, hidden=hidden, from_expression=from_expression)
         self.output_type = "data"
         self.format = format
         self.format_source = format_source
@@ -45,6 +50,7 @@ class ToolOutput(ToolOutputBase):
         self.change_format = []
         self.implicit = implicit
         self.from_work_dir = None
+        self.dataset_collector_descriptions: List[DatasetCollectionDescription] = []
 
     # Tuple emulation
 
@@ -95,6 +101,10 @@ class ToolOutput(ToolOutputBase):
         output.dataset_collector_descriptions = dataset_collector_descriptions_from_output_dict(output_dict)
         return output
 
+    @property
+    def output_discover_patterns(self) -> List[str]:
+        return _merge_dataset_collector_descriptions_patterns(self.dataset_collector_descriptions)
+
 
 class ToolExpressionOutput(ToolOutputBase):
     dict_collection_visible_keys = ('name', 'format', 'label', 'hidden', 'output_type')
@@ -122,15 +132,17 @@ class ToolOutputCollection(ToolOutputBase):
     Represents a HistoryDatasetCollectionAssociation of output datasets produced
     by a tool.
 
-    <outputs>
-      <collection type="list" label="${tool.name} on ${on_string} fasta">
-        <discover_datasets pattern="__name__" ext="fasta" visible="True" directory="outputFiles" />
-      </collection>
-      <collection type="paired" label="${tool.name} on ${on_string} paired reads">
-        <data name="forward" format="fastqsanger" />
-        <data name="reverse" format="fastqsanger"/>
-      </collection>
-    <outputs>
+    .. code-block::
+
+        <outputs>
+        <collection type="list" label="${tool.name} on ${on_string} fasta">
+            <discover_datasets pattern="__name__" ext="fasta" visible="True" directory="outputFiles" />
+        </collection>
+        <collection type="paired" label="${tool.name} on ${on_string} paired reads">
+            <data name="forward" format="fastqsanger" />
+            <data name="reverse" format="fastqsanger"/>
+        </collection>
+        <outputs>
     """
     dict_collection_visible_keys = ['name', 'format', 'label', 'hidden', 'output_type', 'default_format',
                                     'default_format_source', 'default_metadata_source', 'inherit_format', 'inherit_metadata']
@@ -153,7 +165,7 @@ class ToolOutputCollection(ToolOutputBase):
         self.collection = True
         self.default_format = default_format
         self.structure = structure
-        self.outputs = OrderedDict()
+        self.outputs = {}
 
         self.inherit_format = inherit_format
         self.inherit_metadata = inherit_metadata
@@ -173,7 +185,8 @@ class ToolOutputCollection(ToolOutputBase):
         else:
             collection_prototype = self.structure.collection_prototype(inputs, type_registry)
 
-            def prototype_dataset_element_to_output(element, parent_ids=[]):
+            def prototype_dataset_element_to_output(element, parent_ids=None):
+                parent_ids = parent_ids or []
                 name = element.element_identifier
                 format = self.default_format
                 if self.inherit_format:
@@ -194,7 +207,8 @@ class ToolOutputCollection(ToolOutputBase):
                     parent_ids=parent_ids,
                 )
 
-            def prototype_collection_to_output(collection_prototype, parent_ids=[]):
+            def prototype_collection_to_output(collection_prototype, parent_ids=None):
+                parent_ids = parent_ids or []
                 output_parts = []
                 for element in collection_prototype.elements:
                     element_parts = []
@@ -243,6 +257,10 @@ class ToolOutputCollection(ToolOutputBase):
         )
         return rval
 
+    @property
+    def output_discover_patterns(self) -> List[str]:
+        return self.structure.output_discover_patterns
+
 
 class ToolOutputCollectionStructure:
 
@@ -298,10 +316,25 @@ class ToolOutputCollectionStructure:
         )
         return structure
 
+    @property
+    def output_discover_patterns(self) -> List[str]:
+        if not self.dataset_collector_descriptions:
+            return []
+        else:
+            return _merge_dataset_collector_descriptions_patterns(self.dataset_collector_descriptions)
+
+
+def _merge_dataset_collector_descriptions_patterns(dataset_collector_descriptions: List[DatasetCollectionDescription]) -> List[str]:
+    patterns = []
+    for description in dataset_collector_descriptions:
+        patterns.extend(description.discover_patterns)
+    return patterns
+
 
 class ToolOutputCollectionPart:
 
-    def __init__(self, output_collection_def, element_identifier, output_def, parent_ids=[]):
+    def __init__(self, output_collection_def, element_identifier, output_def, parent_ids=None):
+        parent_ids = parent_ids or []
         self.output_collection_def = output_collection_def
         self.element_identifier = element_identifier
         self.output_def = output_def
@@ -311,7 +344,7 @@ class ToolOutputCollectionPart:
     def effective_output_name(self):
         name = self.output_collection_def.name
         part_name = self.element_identifier
-        effective_output_name = "{}|__part__|{}".format(name, part_name)
+        effective_output_name = f"{name}|__part__|{part_name}"
         return effective_output_name
 
     @staticmethod
