@@ -20,9 +20,10 @@ def assert_has_h5_attribute(output_bytes, key, value):
     output_temp = io.BytesIO(output_bytes)
     local_attrs = h5py.File(output_temp, 'r').attrs
     assert key in local_attrs and str(local_attrs[key]) == value, (
-        "Not a HDF5 file or H5 attributes do not match:\n\t{}\n\n\t({} : {})".format(local_attrs.items(), key, value))
+        f"Not a HDF5 file or H5 attributes do not match:\n\t{list(local_attrs.items())}\n\n\t({key} : {value})")
 
 
+# TODO the function actually queries groups. so the function and argument name are misleading
 def assert_has_h5_keys(output_bytes, keys):
     """ Asserts the specified HDF5 output has the given keys."""
     _assert_h5py()
@@ -39,4 +40,4 @@ def assert_has_h5_keys(output_bytes, keys):
     for key in h5_keys:
         if key not in local_keys:
             missing += 1
-    assert missing == 0, "Not a HDF5 file or H5 keys missing:\n\t{}\n\t{}".format(local_keys, h5_keys)
+    assert missing == 0, f"Not a HDF5 file or H5 keys missing:\n\t{local_keys}\n\t{h5_keys}"
