@@ -6,6 +6,7 @@ from galaxy.tool_util.deps.mulled.mulled_build import (
     DEFAULT_BASE_IMAGE,
     DEFAULT_EXTENDED_BASE_IMAGE,
     mull_targets,
+    target_str_to_targets
 )
 from ..util import external_dependency_management
 
@@ -27,3 +28,10 @@ def test_mulled_build_files_cli(tmpdir):
     target = build_target('zlib')
     mull_targets([target], command='build-and-test', singularity=True, singularity_image_dir=singularity_image_dir)
     assert singularity_image_dir.join('zlib').exists()
+
+
+def test_target_str_to_targets():
+    target_str = 'samtools=1.3.1--4,bedtools=2.22'
+    targets = target_str_to_targets(target_str)
+    assert (targets[0].package, targets[0].version, targets[0].build) == ('samtools', '1.3.1', '4')
+    assert (targets[1].package, targets[1].version, targets[1].build) == ('bedtools', '2.22', None)
