@@ -1,6 +1,13 @@
 import pytest
 
-from galaxy.tool_util.deps.mulled.mulled_search import CondaSearch, get_package_hash, GitHubSearch, QuaySearch, run_command, singularity_search
+from galaxy.tool_util.deps.mulled.mulled_search import (
+    CondaSearch,
+    get_package_hash,
+    GitHubSearch,
+    QuaySearch,
+    run_command,
+    singularity_search
+)
 from ..util import external_dependency_management
 
 
@@ -27,12 +34,21 @@ def test_conda_search():
 
 
 @external_dependency_management
-def test_github_search():
+def test_github_recipe_present():
     t = GitHubSearch()
-    search1 = t.process_json(t.get_json("adsfasdf"), "adsfasdf")
-    search2 = t.process_json(t.get_json("bioconductor-gosemsim"), "bioconductor-gosemsim")
-    assert search1 == []
-    assert {'path': 'recipes/bioconductor-gosemsim/meta.yaml', 'name': 'meta.yaml'} in search2
+
+    search_string2expected = {
+        "adsfasdf": False,
+        "bioconductor-gosemsim": True,
+    }
+    for search_string, expected in search_string2expected.items():
+        try:
+            is_recipe_present = t.recipe_present(search_string)
+        except Exception as e:
+            if "API rate limit" in str(e):
+                pytest.skip("Hitting GitHub API rate limit")
+            raise
+        assert is_recipe_present is expected
 
 
 @external_dependency_management
@@ -50,7 +66,10 @@ def test_get_package_hash():
 @external_dependency_management
 def test_singularity_search():
     sing1 = singularity_search('mulled-v2-0560a8046fc82aa4338588eca29ff18edab2c5aa')
+    sing1_versions = {result['version'] for result in sing1}
+    assert {
+        'c17ce694dd57ab0ac1a2b86bb214e65fedef760e-0',
+        'f471ba33d45697daad10614c5bd25a67693f67f1-0',
+        'fc33176431a4b9ef3213640937e641d731db04f1-0'}.issubset(sing1_versions)
     sing2 = singularity_search('mulled-v2-19fa9431f5863b2be81ff13791f1b00160ed0852')
-    assert sing1[0]['version'] in ['c17ce694dd57ab0ac1a2b86bb214e65fedef760e-0', 'fc33176431a4b9ef3213640937e641d731db04f1-0']
-    assert sing1[1]['version'] in ['c17ce694dd57ab0ac1a2b86bb214e65fedef760e-0', 'fc33176431a4b9ef3213640937e641d731db04f1-0']
     assert sing2 == []
